@@ -43,33 +43,6 @@
 #define SEC_ACE_OBJECT_PRESENT           0x00000001 /* thanks for Jim McDonough <jmcd@us.ibm.com> */
 #define SEC_ACE_OBJECT_INHERITED_PRESENT 0x00000002
 
-#define SEC_ACE_FLAG_OBJECT_INHERIT		0x1
-#define SEC_ACE_FLAG_CONTAINER_INHERIT		0x2
-#define SEC_ACE_FLAG_NO_PROPAGATE_INHERIT	0x4
-#define SEC_ACE_FLAG_INHERIT_ONLY		0x8
-#define SEC_ACE_FLAG_INHERITED_ACE		0x10 /* New for Windows 2000 */
-#define SEC_ACE_FLAG_VALID_INHERIT		0xf
-#define SEC_ACE_FLAG_SUCCESSFUL_ACCESS		0x40
-#define SEC_ACE_FLAG_FAILED_ACCESS		0x80
-
-#define SEC_ACE_TYPE_ACCESS_ALLOWED		0x0
-#define SEC_ACE_TYPE_ACCESS_DENIED		0x1
-#define SEC_ACE_TYPE_SYSTEM_AUDIT		0x2
-#define SEC_ACE_TYPE_SYSTEM_ALARM		0x3
-#define SEC_ACE_TYPE_ALLOWED_COMPOUND		0x4
-#define SEC_ACE_TYPE_ACCESS_ALLOWED_OBJECT	0x5
-#define SEC_ACE_TYPE_ACCESS_DENIED_OBJECT     	0x6
-#define SEC_ACE_TYPE_SYSTEM_AUDIT_OBJECT      	0x7
-#define SEC_ACE_TYPE_SYSTEM_ALARM_OBJECT	0x8
-
-#define SEC_DESC_OWNER_DEFAULTED	0x0001
-#define SEC_DESC_GROUP_DEFAULTED	0x0002
-#define SEC_DESC_DACL_PRESENT		0x0004
-#define SEC_DESC_DACL_DEFAULTED		0x0008
-#define SEC_DESC_SACL_PRESENT		0x0010
-#define SEC_DESC_SACL_DEFAULTED		0x0020
-#define SEC_DESC_DACL_TRUSTED		0x0040
-#define SEC_DESC_SERVER_SECURITY	0x0080
 /*
  * New Windows 2000 bits.
  */
@@ -80,11 +53,6 @@
 #define SE_DESC_DACL_PROTECTED		0x1000
 #define SE_DESC_SACL_PROTECTED		0x2000
 
-/* Don't know what this means. */
-#define SEC_DESC_RM_CONTROL_VALID	0x4000
-
-#define SEC_DESC_SELF_RELATIVE		0x8000
-
 /* security information */
 #define OWNER_SECURITY_INFORMATION	0x00000001
 #define GROUP_SECURITY_INFORMATION	0x00000002
@@ -104,25 +72,20 @@
 					PROTECTED_DACL_SECURITY_INFORMATION)
 
 /* SEC_ACCESS */
-typedef struct security_info_info
-{
-	uint32 mask;
-
-} SEC_ACCESS;
+typedef uint32 SEC_ACCESS;
 
 /* SEC_ACE */
-typedef struct security_ace_info
-{
+typedef struct security_ace_info {
 	uint8 type;  /* xxxx_xxxx_ACE_TYPE - e.g allowed / denied etc */
 	uint8 flags; /* xxxx_INHERIT_xxxx - e.g OBJECT_INHERIT_ACE */
 	uint16 size;
 
-	SEC_ACCESS info;
+	SEC_ACCESS access_mask;
 
 	/* this stuff may be present when type is XXXX_TYPE_XXXX_OBJECT */
 	uint32  obj_flags; /* xxxx_ACE_OBJECT_xxxx e.g present/inherited present etc */
-	struct uuid obj_guid;  /* object GUID */
-	struct uuid inh_guid;  /* inherited object GUID */		
+	struct GUID obj_guid;  /* object GUID */
+	struct GUID inh_guid;  /* inherited object GUID */		
         /* eof object stuff */
 
 	DOM_SID trustee;
@@ -134,19 +97,14 @@ typedef struct security_ace_info
 #define ACL_REVISION 0x3
 #endif
 
-#ifndef NT4_ACL_REVISION
-#define NT4_ACL_REVISION 0x2
-#endif
-
 #ifndef _SEC_ACL
 /* SEC_ACL */
-typedef struct security_acl_info
-{
+typedef struct security_acl_info {
 	uint16 revision; /* 0x0003 */
 	uint16 size; /* size in bytes of the entire ACL structure */
 	uint32 num_aces; /* number of Access Control Entries */
 
-	SEC_ACE *ace;
+	SEC_ACE *aces;
 
 } SEC_ACL;
 #define  SEC_ACL_HEADER_SIZE (2 * sizeof(uint16) + sizeof(uint32))
@@ -159,8 +117,7 @@ typedef struct security_acl_info
 
 #ifndef _SEC_DESC
 /* SEC_DESC */
-typedef struct security_descriptor_info
-{
+typedef struct security_descriptor_info {
 	uint16 revision; /* 0x0001 */
 	uint16 type;     /* SEC_DESC_xxxx flags */
 
@@ -172,7 +129,7 @@ typedef struct security_descriptor_info
 	SEC_ACL *dacl; /* user ACL */
 	SEC_ACL *sacl; /* system ACL */
 	DOM_SID *owner_sid; 
-	DOM_SID *grp_sid;
+	DOM_SID *group_sid;
 
 } SEC_DESC;
 #define  SEC_DESC_HEADER_SIZE (2 * sizeof(uint16) + 4 * sizeof(uint32))
@@ -181,8 +138,7 @@ typedef struct security_descriptor_info
 
 #ifndef _SEC_DESC_BUF
 /* SEC_DESC_BUF */
-typedef struct sec_desc_buf_info
-{
+typedef struct sec_desc_buf_info {
 	uint32 max_len;
 	uint32 ptr;
 	uint32 len;
@@ -239,6 +195,7 @@ typedef struct standard_mapping {
 
 /* Combinations of standard masks. */
 #define STANDARD_RIGHTS_ALL_ACCESS	STD_RIGHT_ALL_ACCESS /* 0x001f0000 */
+#define STANDARD_RIGHTS_MODIFY_ACCESS	STD_RIGHT_READ_CONTROL_ACCESS /* 0x00020000 */
 #define STANDARD_RIGHTS_EXECUTE_ACCESS	STD_RIGHT_READ_CONTROL_ACCESS /* 0x00020000 */
 #define STANDARD_RIGHTS_READ_ACCESS	STD_RIGHT_READ_CONTROL_ACCESS /* 0x00020000 */
 #define STANDARD_RIGHTS_WRITE_ACCESS \
@@ -287,10 +244,23 @@ typedef struct standard_mapping {
 
 #define GENERIC_RIGHTS_FILE_EXECUTE \
 		(STANDARD_RIGHTS_EXECUTE_ACCESS	| \
+		STD_RIGHT_SYNCHRONIZE_ACCESS	| \
 		SA_RIGHT_FILE_READ_ATTRIBUTES	| \
 		SA_RIGHT_FILE_EXECUTE)            
 
-		
+#define GENERIC_RIGHTS_FILE_MODIFY \
+		(STANDARD_RIGHTS_MODIFY_ACCESS	| \
+		STD_RIGHT_SYNCHRONIZE_ACCESS	| \
+		STD_RIGHT_DELETE_ACCESS		| \
+		SA_RIGHT_FILE_WRITE_ATTRIBUTES	| \
+		SA_RIGHT_FILE_READ_ATTRIBUTES	| \
+		SA_RIGHT_FILE_EXECUTE		| \
+		SA_RIGHT_FILE_WRITE_EA		| \
+		SA_RIGHT_FILE_READ_EA		| \
+		SA_RIGHT_FILE_APPEND_DATA	| \
+		SA_RIGHT_FILE_WRITE_DATA	| \
+		SA_RIGHT_FILE_READ_DATA)
+
 /* SAM server specific access rights */
 
 #define SA_RIGHT_SAM_CONNECT_SERVER	0x00000001
