@@ -31,6 +31,21 @@
 #define	PIPE		"\\PIPE\\"
 #define	PIPELEN		strlen(PIPE)
 
+#define MAX_PIPE_NAME_LEN	24
+
+/* PIPE/<name>/<pid>/<pnum> */
+#define PIPEDB_KEY_FORMAT "PIPE/%s/%u/%d"
+
+struct pipe_dbrec {
+	struct process_id pid;
+	int pnum;
+	uid_t uid;
+
+	char name[MAX_PIPE_NAME_LEN];
+	fstring	user;
+};
+
+
 extern struct pipe_id_info pipe_names[];
 
 /****************************************************************************
@@ -121,6 +136,7 @@ int reply_open_pipe_and_X(connection_struct *conn,
 int reply_pipe_write(char *inbuf,char *outbuf,int length,int dum_bufsize)
 {
 	smb_np_struct *p = get_rpc_pipe_p(inbuf,smb_vwv0);
+	uint16 vuid = SVAL(inbuf,smb_uid);
 	size_t numtowrite = SVAL(inbuf,smb_vwv1);
 	int nwritten;
 	int outsize;
@@ -130,6 +146,10 @@ int reply_pipe_write(char *inbuf,char *outbuf,int length,int dum_bufsize)
 		return(ERROR_DOS(ERRDOS,ERRbadfid));
 	}
 
+	if (p->vuid != vuid) {
+		return ERROR_NT(NT_STATUS_INVALID_HANDLE);
+	}
+
 	data = smb_buf(inbuf) + 3;
 
 	if (numtowrite == 0) {
@@ -161,6 +181,7 @@ int reply_pipe_write(char *inbuf,char *outbuf,int length,int dum_bufsize)
 int reply_pipe_write_and_X(char *inbuf,char *outbuf,int length,int bufsize)
 {
 	smb_np_struct *p = get_rpc_pipe_p(inbuf,smb_vwv2);
+	uint16 vuid = SVAL(inbuf,smb_uid);
 	size_t numtowrite = SVAL(inbuf,smb_vwv10);
 	int nwritten = -1;
 	int smb_doff = SVAL(inbuf, smb_vwv11);
@@ -172,6 +193,10 @@ int reply_pipe_write_and_X(char *inbuf,char *outbuf,int length,int bufsize)
 		return(ERROR_DOS(ERRDOS,ERRbadfid));
 	}
 
+	if (p->vuid != vuid) {
+		return ERROR_NT(NT_STATUS_INVALID_HANDLE);
+	}
+
 	data = smb_base(inbuf) + smb_doff;
 
 	if (numtowrite == 0) {
@@ -274,6 +299,6 @@ int reply_pipe_close(connection_struct *conn, char *inbuf,char *outbuf)
 	if (!close_rpc_pipe_hnd(p)) {
 		return ERROR_DOS(ERRDOS,ERRbadfid);
 	}
-
+	
 	return(outsize);
 }
